/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef OPENLDEV_ENVIRONMENT_H
#define OPENLDEV_ENVIRONMENT_H

/*! \file openldev-settings.h
    \brief EnvironmentSettings
*/

#include <gconf/gconf-client.h>
#include <cstdlib>
#include <glib.h>
#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

/*! \brief General settings defined by the user's preferences. */
struct EnvironmentSettings
{
  /*! Client that all of the settings are loaded from and saved to. */
  GConfClient *client;

  /*! Number of minutes between OpenLDev saving all of the open files. Set to 0 to disable. */
  gint auto_save;
  /*! Width in pixels of the project sidebar. */
  gint sidebar_width;
  /*! Height in pixels of the application window. */
  gint height;
  /*! Height in pixels of the build output manager. */
  gint outputmanager_height;
  /*! Number of spaces inserted when the tab key is pressed. */
  gint tab_size;
  /*! Number of undoable actions to remember for each editor tab. */
  gint undo;
  /*! Width in pixels of the application window. */
  gint width;
  
  /*! TRUE if auto indentation is enabled. */
  gboolean auto_indent;
  /*! TRUE if the last opened project is loaded on application load. */
  gboolean auto_load;
  /*! TRUE if the matching bracket is highlighted in the editor. */
  gboolean brackets;
  /*! TRUE if line numbers are shown. */
  gboolean line_numbers;
  /*! TRUE if spaces are inserted instead of tab characters. */
  gboolean replace_tabs;
  /*! TRUE if a line should show the right margin at 80 characters. */
  gboolean right_margin;
  /*! TRUE if wrapping should be enabled in each editor tab. */
  gboolean wrap;
  /*! TRUE if words should be split when wrapping in each editor tab. */
  gboolean wrap_split;
  
  /*! The color used for an error in build output (e.g. #FF0000). */
  string error;
  /*! The font face used in each editor tab. */
  string font_face;
  /*! A string representation of the font size used in each tab (e.g. 12). */
  string font_size;
  /*! Full file name of the last opened project. This will be loaded on application start if auto_load is TRUE. */
  string last_project;
  /*! The color used for an warning in build output (e.g. #FF00FF). */
  string warning;
  
  /*! List of the five most recently opened files in the editor. */
  GSList *recent_files;
  /*! List of enabled plugins. These plugins are the only ones enabled on the next application start. */
  GSList *plugins;
};

/*! This will create a new EnvironmentSettings project with the settings loaded from the user's GConf database.
  \return A new EnvironmentSettings object.
*/
EnvironmentSettings* openldev_environment_settings_new ();

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if auto indentation should be enabled.
*/
void openldev_environment_set_auto_indent (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if the last opened project should be loaded when OpenLDev is restarted.
*/
void openldev_environment_set_auto_load (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if the matching bracket should be highlighted.
*/
void openldev_environment_set_brackets (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if line numbers should be displayed in the editor.
*/
void openldev_environment_set_line_numbers (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if tabs should be replaced by spaces.
*/
void openldev_environment_set_replace_tabs (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if the right margin should be shown at 80 characters.
*/
void openldev_environment_set_right_margin (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if text wrapping should be enabled.
*/
void openldev_environment_set_wrap (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value TRUE if words should be split when wrapped around the edge of the buffer.
*/
void openldev_environment_set_wrap_split (EnvironmentSettings *settings, gboolean value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The number of minutes OpenLDev should wait before saving all of the open and modified files. Set to 0 to disable.
*/
void openldev_environment_set_auto_save (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The width in pixels of the project file sidebar.
*/
void openldev_environment_set_sidebar_width (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The height in pixels of the OpenLDev application window.
*/
void openldev_environment_set_height (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The height in pixels of the output manager.
*/
void openldev_environment_set_outputmanager_height (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The number of spaces that should be inserted when the tab key is pressed.
*/
void openldev_environment_set_tab_size (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The number of undoable actions to remember for each editor tab.
*/
void openldev_environment_set_undo (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The width in pixels of the OpenLDev application window.
*/
void openldev_environment_set_width (EnvironmentSettings *settings, int value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The 6-digit hexadecimal color used for error messages in the build output (e.g. #FF0000).
*/
void openldev_environment_set_error (EnvironmentSettings *settings, const char *value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The font face string to be used in each editor tab.
*/
void openldev_environment_set_font_face (EnvironmentSettings *settings, const char *value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value A string representation of the font size to be used in each editor tab (e.g. 12).
*/
void openldev_environment_set_font_size (EnvironmentSettings *settings, const char *value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The full path of the last opened project.
*/
void openldev_environment_set_last_project (EnvironmentSettings *settings, const char *value);

/*!
  \param settings A non-NULL EnvironmentSettings object.
  \param value The 6-digit hexadecimal color used for warning messages in the build output (e.g. #FF00FF).
*/
void openldev_environment_set_warning (EnvironmentSettings *settings, const char *value);

/*! Add a file that was just opened to the recent files list. If there are 5 files already on the list, the last one will be deleted.
  \param settings A non-NULL EnvironmentSettings object.
  \param file The full path of a file that is currently opened.
*/
void openldev_environment_add_recent_file (EnvironmentSettings *settings, const char *file);

/*! Get the short file name of a plugin located at position id. This helps when trying to make a menu list of recent files.
  \param settings A non-NULL EnvironmentSettings object.
  \param id The id of the recent file to obtain, from 0 to 4.
  \return The short name of a recent file located at id.
*/
const char* openldev_environment_get_recent_short (EnvironmentSettings *settings, int id);

/*! Check whether a plugin is currently enabled. You must pass the short name of the plugin, not the full file name.
  \param settings A non-NULL EnvironmentSettings object.
  \param plugin A short plugin name.
  \return TRUE if the plugin is enabled.
*/
gboolean openldev_environment_get_plugin (EnvironmentSettings *settings, const char *plugin);

/*! Remove a plugin from the list of enabled. The changes will be saved to GConf so it can be disabled in the future.
  \param settings A non-NULL EnvironmentSettings object.
  \param plugin A short plugin name.
*/
void openldev_environment_remove_plugin (EnvironmentSettings *settings, const char *plugin);

/*! Add a plugin to the list of enabled. The changes will be saved to GConf so it can be enabled in the future.
  \param settings A non-NULL EnvironmentSettings object.
  \param plugin A short plugin name.
*/
void openldev_environment_add_plugin (EnvironmentSettings *settings, const char *plugin);


#endif
