/***************************************************************************
 *	 Copyright (C) 2006 by Uwe Grawert                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.              *
 ***************************************************************************/

#include "openldev-xml-parser.h"

// Create a new xml document with parameter xml_file where to save it
OpenLDevXmlDocument* openldev_xml_document_new (const string& xml_file)
{
  OpenLDevXmlDocument *newXmlDocument = new OpenLDevXmlDocument;

  newXmlDocument->xml_file = xml_file;
  newXmlDocument->xml_doc  = xmlNewDoc((xmlChar*)"1.0");

  return newXmlDocument;
}

// Open an xml document from xml_file
OpenLDevXmlDocument* openldev_xml_document_open (const string& xml_file)
{
  xmlInitParser();
  OpenLDevXmlDocument *open_xml_doc = new OpenLDevXmlDocument();
  open_xml_doc->xml_doc = xmlReadFile (xml_file.c_str(), NULL, 0);

  if (open_xml_doc->xml_doc != NULL)
    open_xml_doc->xml_file = xml_file;
  // Error reading the project file
  else
  { 
    delete open_xml_doc; 
    open_xml_doc = NULL; 
  }

  xmlCleanupParser();
  return open_xml_doc;
}

// Save an xml document
void openldev_xml_document_save (OpenLDevXmlDocument** xml_document)
{
  OpenLDevXmlDocument *xml_doc_ptr = *xml_document;

  xmlCleanupParser();
  if (xml_doc_ptr != NULL)
    xmlSaveFormatFileEnc (xml_doc_ptr->xml_file.c_str(),
                          xml_doc_ptr->xml_doc, "UTF-8", 1);
}

// Close an xml document
void openldev_xml_document_close (OpenLDevXmlDocument** xml_document)
{
  OpenLDevXmlDocument *xml_doc_ptr = *xml_document;

  if (xml_doc_ptr != NULL)
  {
		xmlKeepBlanksDefault (0);
    xmlSaveFormatFileEnc (xml_doc_ptr->xml_file.c_str(),
                          xml_doc_ptr->xml_doc, "UTF-8", 1);
    xmlFreeDoc (xml_doc_ptr->xml_doc);
    delete xml_doc_ptr; *xml_document = NULL;
  }
}

// Change the name under which the xml file will be saved
void openldev_xml_document_change_filename (OpenLDevXmlDocument* xml_document,
                                            const string& new_filename)
{
  if(xml_document != NULL)
    xml_document->xml_file = new_filename;
}

// Set a new root element. The old structure of the xml document will be lost
void openldev_xml_document_set_root_element (OpenLDevXmlDocument* xml_document,
                                             const string& element_name)
{
  if (xml_document != NULL)
  {
    xmlNodePtr new_node = xmlNewNode(NULL, (xmlChar*)element_name.c_str());
    xmlDocSetRootElement(xml_document->xml_doc, new_node);
    xmlFree(new_node);
  }
}

// Set an attribute in the root element
void 
openldev_xml_document_set_root_attribute (OpenLDevXmlDocument* xml_document,
                                          const string& attribute_name,
                                          const string& value)
{
  if(xml_document != NULL)
  {
    xmlNodePtr root_node = xmlDocGetRootElement(xml_document->xml_doc);
    xmlSetProp(root_node, (xmlChar*)attribute_name.c_str(),
               (xmlChar*)value.c_str());
  }
}

// Remove an attribute from the root element
void
openldev_xml_document_remove_root_attribute (OpenLDevXmlDocument* xml_document,
                                             const string& attribute_name)
{
  if(xml_document != NULL)
  {
    xmlNodePtr root_node = xmlDocGetRootElement(xml_document->xml_doc);
    xmlAttrPtr del_attr  = xmlHasProp(root_node,
                                      (xmlChar*)attribute_name.c_str());
    xmlRemoveProp(del_attr);
  }
}

// Search for the element with name starting at root_node
xmlNodePtr find_xmlNode_by_name (xmlNodePtr root_node,
                                 const xmlChar* element_name)
{
  xmlNodePtr key_node = NULL;

  root_node = root_node->children;

  while(root_node != NULL)
  {
    // compare elements name with element_name
    if((root_node->type == XML_ELEMENT_NODE) &&
        !xmlStrcmp(root_node->name, element_name))
      return root_node;
    else
    {
      // process recursively the child elements
      if(root_node->children)
        key_node = find_xmlNode_by_name(root_node->children, element_name);
      if(key_node != NULL)
        return key_node;
    }

    root_node = root_node->next;
  }
  // nothing found
  return NULL;
}

// Set an element with value. Value will be changed if element exists
gboolean openldev_xml_document_set_element (OpenLDevXmlDocument* xml_document,
                                            const string& element_name,
                                            const string& value)
{
  if(xml_document == NULL)
    return FALSE;

  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr key_node = find_xmlNode_by_name (root_node,
                                             (xmlChar*)element_name.c_str());
  
  if(key_node != NULL)
    xmlNodeSetContent(key_node, (xmlChar*)value.c_str());
  else
  {
    xmlNodePtr new_node = xmlNewNode(NULL, (xmlChar*)element_name.c_str());
    xmlNodeSetContent(new_node, (xmlChar*)value.c_str());
    xmlAddChild(root_node, new_node);
  }

  return TRUE;
}

// Add new element as child to parent element
gboolean openldev_xml_document_add_element (OpenLDevXmlDocument* xml_document,
                                            const string& element_name,
                                            const string& value)
{
  if(xml_document == NULL)
    return FALSE;

  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr new_node = xmlNewNode(NULL, (xmlChar*)element_name.c_str());
  xmlNodeSetContent(new_node, (xmlChar*)value.c_str());
  xmlAddChild(root_node, new_node);

  return TRUE;
}

// Set an attribute with value. If attribute is missing it will be created
gboolean openldev_xml_document_set_attribute (OpenLDevXmlDocument* xml_document,
                                              const string& parent_name,
                                              const string& element_name,
                                              const string& attribute_name,
                                              const string& value)
{
  if(xml_document == NULL)
    return FALSE;

  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr parent_node = find_xmlNode_by_name(root_node,
                                                (xmlChar*)parent_name.c_str());
  xmlNodePtr key_node    = find_xmlNode_by_name(parent_node,
                                                (xmlChar*)element_name.c_str());

  if(key_node == NULL)
    return FALSE;

  xmlSetProp(key_node, (xmlChar*)attribute_name.c_str(),
             (xmlChar*)value.c_str());

  return TRUE;
}

// Return the value for element named element_name
string openldev_xml_document_element_value (OpenLDevXmlDocument* xml_document,
                                            const string& element_name)
{
  string ret_string;

  if(xml_document == NULL)
    return ret_string;

  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr key_node    = find_xmlNode_by_name(root_node,
                                              (xmlChar*)element_name.c_str());
  xmlChar*   result      = xmlNodeGetContent(key_node);

  if(result != NULL)
  {
    ret_string = string((char*) result);
    xmlFree(result);
  }

  return ret_string;
}

// Return a list of property values of all elements named element_name
vector<string>
openldev_xml_document_element_values (OpenLDevXmlDocument* xml_document,
                                      const string& element_name)
{
  vector<string> ret_vector;

  if(xml_document == NULL)
    return ret_vector;

  xmlChar *value, *ckey  = (xmlChar*)element_name.c_str();
  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr key_node    = find_xmlNode_by_name(root_node, ckey);

  // go through all element siblings
  for(xmlNodePtr itr = key_node; itr != NULL; itr = itr->next)
  {
    value = NULL;
    // search for element siblings with same name
    if((itr->type == XML_ELEMENT_NODE) && !xmlStrcmp(itr->name, ckey))
    {
      value = xmlNodeGetContent(itr);
      if( value != NULL )
      {
        ret_vector.push_back(string((char*)value));
        xmlFree(value);
      }
    }
  }

  return ret_vector;
}

// Return the value of an attribute from element
string openldev_xml_document_root_attribute (OpenLDevXmlDocument* xml_document,
                                             const string& attribute_name)
{
  string ret_string;

  if(xml_document == NULL)
    return ret_string;

  xmlChar *cattr_val;
  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  cattr_val = xmlGetProp(root_node, (xmlChar*)attribute_name.c_str());

  if(cattr_val != NULL)
  {
    ret_string = string((char*)cattr_val);
    xmlFree(cattr_val);
  }

  return ret_string;
}

// Return the value of an attribute from element
string openldev_xml_document_attribute_value (OpenLDevXmlDocument* xml_document,
                                              const string& parent_name,
                                              const string& element_name,
                                              const string& attribute_name)
{
  string ret_string;

  if(xml_document == NULL)
    return ret_string;

  xmlChar *cattr_val;
  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr parent_node = find_xmlNode_by_name(root_node,
                                                 (xmlChar*)parent_name.c_str());
  xmlNodePtr key_node    = find_xmlNode_by_name(parent_node,
                                                (xmlChar*)element_name.c_str());

  cattr_val = xmlGetProp(key_node, (xmlChar*)attribute_name.c_str());

  if(cattr_val != NULL)
  {
    ret_string = string((char*)cattr_val);
    xmlFree(cattr_val);
  }

  return ret_string;
}

// Remove a child element with given value from parent element
void openldev_xml_document_remove_element (OpenLDevXmlDocument* xml_document,
                                   		 		 const string& element_name,
                                   				 const string& value)
{
  if(xml_document == NULL)
    return;

  xmlChar* ckey = (xmlChar*)element_name.c_str();
  xmlChar* cval = (xmlChar*)value.c_str();
  xmlNodePtr root_node = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr remove_node, itr;

  // remove element, regardless of the value
  if(value.empty())
  {
    // go through all child elements of parent_node
    for(itr = root_node->children; itr != NULL; itr = itr->next)
    {
      // compare child nodes name with element_name
      if((itr->type == XML_ELEMENT_NODE) && !xmlStrcmp(itr->name, ckey))
      {
        remove_node = itr;
        itr = itr->next;
        xmlUnlinkNode(remove_node);
        xmlFreeNode(remove_node);
      }
    }
  }
  // remove element with same value
  else
  {
    bool must_break = false;
    
    // go through all child elements of parent_node
    for(itr = root_node->children; itr != NULL; itr = itr->next)
    {
      if (itr->next == NULL)
        must_break = true;
    
      // compare child nodes name with element_name
      if((itr->type == XML_ELEMENT_NODE) && !xmlStrcmp(itr->name, ckey))
      {
        // compare child nodes content with value
        if(!xmlStrcmp(xmlNodeGetContent(itr), cval))
        {
          remove_node = itr;
          itr = itr->next;
          xmlUnlinkNode(remove_node);
          xmlFreeNode(remove_node);
        }
      }
      
      if (must_break)
        break;
    }
  }
}

// Remove all children from parent element
void openldev_xml_document_remove_children (OpenLDevXmlDocument* xml_document,
                                            const string& parent_element)
{
  if(xml_document == NULL)
    return;

  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr parent_node = find_xmlNode_by_name(root_node,
                                                (xmlChar*)parent_element.c_str());
  xmlNodePtr remove_node, itr;

  if(parent_node == NULL)
    return;

  // go through all child elements of parent_node and remove children
  for(itr = parent_node->children; itr != NULL; itr = itr->next)
  {
      remove_node = itr;
      itr = itr->next;
      xmlUnlinkNode(remove_node);
      xmlFreeNode(remove_node);
  }
}

// Remove an attribute from an element
void openldev_xml_document_remove_attribute (OpenLDevXmlDocument* xml_document,
                                             const string& parent_name,
                                             const string& element_name,
                                             const string& attribute_name)
{
  xmlAttrPtr del_attr;
  xmlNodePtr root_node   = xmlDocGetRootElement(xml_document->xml_doc);
  xmlNodePtr parent_node = find_xmlNode_by_name(root_node,
                                                (xmlChar*)parent_name.c_str());
  xmlNodePtr key_node    = find_xmlNode_by_name(parent_node,
                                                (xmlChar*)element_name.c_str());
  
  del_attr = xmlHasProp(key_node, (xmlChar*)attribute_name.c_str());
  xmlRemoveProp(del_attr);
}

