/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"

enum {
  PROJECT_OPEN,
  SIGNAL_COUNT
};

// Initialization Functions
void openldev_initialize_objects (OpenLDev*);
void openldev_populate_window (OpenLDev*);
void openldev_create_menus_and_toolbar (OpenLDev*);
void openldev_populate_recent_files (OpenLDev*);

// Signal Handler declarations
void openldev_on_recent_file_activated (GtkMenuItem*, OpenLDev*);

// Class initialization functions
static void openldev_class_init (OpenLDevClass *klass);
static void openldev_init (OpenLDev *openldev);

static guint openldev_signals [SIGNAL_COUNT];

GType
openldev_get_type (void)
{
  static GType openldev_type = 0;

  if (!openldev_type)
  {
    static const GTypeInfo openldev_info =
    {
	    sizeof (OpenLDevClass),
	    NULL, NULL,
	    (GClassInitFunc) openldev_class_init,
      NULL, NULL,
      sizeof (OpenLDev), 0,
	    (GInstanceInitFunc) openldev_init,
    };

    openldev_type = g_type_register_static (GTK_TYPE_WINDOW, "OpenLDev", 
                                            &openldev_info, (GTypeFlags) 0);
  }

  return openldev_type;
}

static void
openldev_class_init (OpenLDevClass *klass)
{
  openldev_signals[PROJECT_OPEN] = g_signal_new ("project-open",
					                         G_TYPE_FROM_CLASS (klass),
	                                 G_SIGNAL_RUN_LAST,
	                                 G_STRUCT_OFFSET (OpenLDevClass, openldev),
                                   NULL, NULL,                
					                         g_cclosure_marshal_VOID__VOID,
                                   G_TYPE_NONE, 0);
}

static void
openldev_init (OpenLDev *openldev)
{
  // Set properties of main window
  gtk_window_set_title (GTK_WINDOW (openldev), g_strconcat ("OpenLDev ", VERSION, NULL));
  
  // Create settings objects
  openldev->project = openldev_project_settings_new ("");
  openldev->env = openldev_environment_settings_new ();
  openldev->editor = openldev_editor_new ();
  
  // Call functions to add main window widgets & show them all
  gtk_window_set_default_size (GTK_WINDOW (openldev), openldev->env->width,
                               openldev->env->height);
  
  openldev_initialize_objects (openldev);
  openldev->pluginlist = openldev_load_plugins_all (openldev);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output), 0);
  openldev_manage_menu_items (openldev);
  openldev_initialize_gconf (openldev);
}

GtkWidget*
openldev_new ()
{
  return GTK_WIDGET (g_object_new (openldev_get_type (), NULL));
}

void 
openldev_initialize_objects (OpenLDev *openldev)
{
  openldev->vpaned = gtk_vpaned_new ();
  openldev->hpaned = gtk_hpaned_new ();
  openldev->vbox = gtk_vbox_new (FALSE, 0);
  openldev->statusbar = openldev_status_bar_new ();
  openldev->output = output_manager_new ();
  openldev->files = file_manager_new ();
  openldev->sidebar = sidebar_new ();
  openldev->findtoolbar = openldev_find_toolbar_new ();
  openldev->gototoolbar = openldev_goto_toolbar_new ();
  openldev->menus = menu_handle_new ();
  
  gtk_widget_set_size_request (FILE_BROWSER (SIDEBAR (openldev->sidebar)->filebrowser)->swin, 
                               openldev->env->sidebar_width, 0);
  gtk_widget_set_size_request (OUTPUT (openldev->output)->terminal, 0, 
                               openldev->env->outputmanager_height);
  
  // Add all of the widgets & show them
  openldev_populate_window (openldev);
  gtk_widget_show_all (GTK_WIDGET (openldev));
  
  openldev_find_toolbar_show_error (openldev->findtoolbar, FALSE);
  openldev_goto_toolbar_show_error (openldev->gototoolbar, FALSE);
  gtk_widget_hide (openldev->findtoolbar->hbox);
  gtk_widget_hide (openldev->gototoolbar->hbox);
}

void 
openldev_populate_window (OpenLDev *openldev)
{
  openldev_create_menus_and_toolbar (openldev);
  
  // Add menu & toolbar to main vbox
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->menus->menubar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->menus->toolbar, FALSE, FALSE, 0);
  gtk_paned_pack1 (GTK_PANED (openldev->hpaned), openldev->sidebar, FALSE, TRUE);
  gtk_paned_pack2 (GTK_PANED (openldev->hpaned), openldev->files, TRUE, TRUE);
  gtk_paned_pack1 (GTK_PANED (openldev->vpaned), openldev->hpaned, TRUE, TRUE);
  gtk_paned_pack2 (GTK_PANED (openldev->vpaned), openldev->output, FALSE, TRUE);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->vpaned, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->gototoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->findtoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (openldev->vbox), openldev->statusbar->appbar, FALSE, FALSE, 0);

  // Add v' & h' boxes to the main window
  gtk_container_add (GTK_CONTAINER (openldev), openldev->vbox);
}

void
openldev_populate_treeview (OpenLDev *openldev)
{ 
  file_browser_populate (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))),
           openldev_project_settings_get_location (openldev->project).c_str());
}

void 
openldev_create_menus_and_toolbar (OpenLDev *openldev)
{
  // Create menu actions and icons
  menu_handle_create_icon_factory ();

  gtk_action_group_add_actions (openldev->menus->group, entries, 
                                NUM_ENTRIES, (gpointer) openldev);
  gtk_action_group_add_toggle_actions (openldev->menus->group, toggle_entries, 
                                       NUM_ENTRIES_TOGGLE, (gpointer) openldev);
  menu_handle_uimanager_init (openldev->menus);
  gtk_window_add_accel_group (GTK_WINDOW (openldev), 
                              gtk_ui_manager_get_accel_group (openldev->menus->uimanager));

  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/File/FileClose", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Project", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Build", FALSE);

  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget 
                 (openldev->menus->uimanager, "/MenuBar/View/ViewLineNumbering")), 
                 openldev->env->line_numbers);
  
  openldev_populate_recent_files (openldev);
}

void
openldev_populate_recent_files (OpenLDev *openldev)
{
  string recent;

  // Hide the recent files until they are needed or populate them
  for (int i=0; i < 5; i++)
  {
    gchar buff[2];
    sprintf (buff, "%d", i+1);
    recent = string ("/MenuBar/File/FileRecent") + buff;
    
    if (g_slist_length (openldev->env->recent_files) <= i)
      gtk_widget_hide (gtk_ui_manager_get_widget (openldev->menus->uimanager, recent.c_str()));
    else
      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (openldev->menus->uimanager, recent.c_str())))),
                          g_strconcat (buff, ". ", openldev_environment_get_recent_short 
                          (openldev->env, i), NULL));
    
    g_signal_connect (G_OBJECT (gtk_ui_manager_get_widget 
                      (openldev->menus->uimanager, recent.c_str())), "activate",
                      G_CALLBACK (openldev_on_recent_file_activated), 
                      (gpointer) openldev);
  }
}

// Quit the program
void 
openldev_file_quit (GtkMenuItem* menuitem, 
                    OpenLDev *openldev) 
{ 
  gboolean is_modified = FALSE;
  string label;
  
  for (int i=0; i < FILE_MANAGER (openldev->files)->tab_count; i++)
  {
    if (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
    gtk_widget_destroy (GTK_WIDGET (openldev));
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
    gtk_widget_destroy (GTK_WIDGET (openldev));

}

void 
openldev_on_recent_file_activated (GtkMenuItem *item,
                                   OpenLDev *openldev)
{
  GtkWidget *label = gtk_bin_get_child (GTK_BIN (item));
  string text = gtk_label_get_text (GTK_LABEL (label));
  int choice = text[0] - 49;
  
  if (choice >= 0 && choice < 5)
    openldev_open_file (openldev, (gchar*) g_slist_nth_data 
                        (openldev->env->recent_files, choice));
}

void 
openldev_open_file_browser (GtkMenuItem *item,
                            OpenLDev *openldev)
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *fn;
    gtk_tree_model_get (model, &iter, 1, &fn, -1);
   
    // If the text is not a group title, open it
    string file = g_strconcat (FILE_BROWSER (sidebar_get_file_browser 
                               (SIDEBAR (openldev->sidebar)))->base,
                               FILE_BROWSER (sidebar_get_file_browser
                               (SIDEBAR (openldev->sidebar)))->current,
                               fn, NULL);
    
    if (string(fn) == "..")
      file_browser_move_up (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))));
    else if (g_file_test (file.c_str(), G_FILE_TEST_IS_DIR))
      file_browser_move (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))), fn);
    else
      openldev_open_file (openldev, (gchar*) file.c_str());
      
    g_free (fn);
  }
}

void 
openldev_jump_to_build_output (GtkMenuItem *item, 
                               OpenLDev *openldev)
{ 
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                         (OUTPUT (openldev->output)->build));
  GtkTreeModel *model;
  GtkTreePath *path = gtk_tree_path_new ();
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    string dstr;
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
    
    // It the output is an error or a warning..
    if (openldev_is_error_or_warning (file))
    {
      gchar *dir;
      string temp = file;
      linenumber = openldev_get_file_name_and_line (temp) - 1;
      
      do
      {
        path = gtk_tree_model_get_path (model, &iter);
        gtk_tree_path_prev (path);
        gtk_tree_model_get_iter (model, &iter, path);
        gtk_tree_model_get (model, &iter, 0, &dir, -1);
        dstr = string(dir);
      } while (dstr.find ("make[", 0) == string::npos);
        
      dstr = dir;
      dstr.erase (0, dstr.find ("Entering", 0) + 20);
      dstr = dstr.substr (0, dstr.find ("'", 0));

      openldev_open_file (openldev, g_strconcat (dstr.c_str(), "/", temp.c_str(), NULL));
        
      // Move the cursor to the line of the error & scroll to the cursor
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER (file_manager_get_current_buffer
                                              (FILE_MANAGER (openldev->files)));
      GtkTextIter iter;

      gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert(buffer), &iter);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
      file_manager_scroll_to_mark (FILE_MANAGER (openldev->files));
      gtk_widget_grab_focus (GTK_WIDGET (file_manager_get_current_view 
                             (FILE_MANAGER (openldev->files))));
      g_free (dir);
    }
      
    g_free (file);
  }
}

void 
openldev_jump_to_search_output (GtkMenuItem *item, 
                                OpenLDev *openldev)
{ 
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                         (OUTPUT (openldev->output)->search));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
      
    // Open the file the error was found in
    string temp = file;
    linenumber = openldev_get_file_name_and_line (temp) - 1;
    file = g_strconcat (openldev_project_settings_get_location (openldev->project).c_str(), 
                        temp.c_str(), NULL);
    openldev_open_file (openldev, file);
    
    // Move the cursor to the line of the error & scroll to the cursor
    GtkTextIter iter;
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER (file_manager_get_current_buffer 
                                            (FILE_MANAGER (openldev->files)));
      
    gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert(buffer), &iter);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
    file_manager_scroll_to_mark (FILE_MANAGER (openldev->files));
    gtk_widget_grab_focus (GTK_WIDGET (file_manager_get_current_view 
                           (FILE_MANAGER (openldev->files))));
     
    g_free (file);
  }
}

// Menu handler functions and signal handlers that do nothing but redirect

void openldev_on_find_and_replace (GtkWidget *widget, OpenLDev *openldev)
{ openldev_find_and_replace (openldev); }

void openldev_refresh_file_browser (GtkMenuItem *item, OpenLDev *openldev)
{ file_browser_refresh (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar)))); }

void openldev_clear_build_output (GtkMenuItem *item, OpenLDev *openldev)
{ output_build_clear (OUTPUT_BUILD (OUTPUT (openldev->output)->build)); }

void openldev_clear_search_output (GtkMenuItem *item, OpenLDev *openldev)
{ output_list_clear (LIST (OUTPUT (openldev->output)->search)); }

void openldev_copy_output (GtkMenuItem *item, OpenLDev *openldev)
{ openldev_menu_edit_copy (openldev); }

void openldev_file_close_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_close_project (openldev); }

void openldev_file_new_file (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_new_file (openldev); }

void openldev_file_new_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_new_project (openldev); }

void openldev_file_open (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_open (openldev); }

void openldev_file_revert (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_revert (openldev); }

void openldev_file_save (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save (openldev); }

void openldev_file_save_all (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save_all (openldev); }

void openldev_file_save_as (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save_as (openldev); }

void openldev_file_print (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_print (openldev); }

void openldev_file_print_preview (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_print_preview (openldev); }

void openldev_edit_copy (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_copy (openldev); }

void openldev_edit_cut (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_cut (openldev); }

void openldev_edit_deselect (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_deselect (openldev); }

void openldev_edit_paste (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_paste (openldev); }

void openldev_edit_preferences (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_preferences (openldev); }

void openldev_edit_redo (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_redo (openldev); }

void openldev_edit_select_all (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_select_all (openldev); }

void openldev_edit_undo (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_undo (openldev); }

void openldev_edit_find (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_find (openldev); }

void openldev_edit_find_next (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_find_next (openldev); }

void openldev_edit_goto_line (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_goto_line (openldev); }

void openldev_project_info (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_project_info (openldev); }

void openldev_project_preferences (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_project_preferences (openldev); }

void openldev_build_auto_generate (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_auto_generate (openldev); }

void openldev_build_clean (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_clean (openldev); }

void openldev_build_compile (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_compile (openldev); }

void openldev_build_configure (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_configure (openldev); }

void openldev_build_execute (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_execute (openldev); }

void openldev_build_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_project (openldev); }

void openldev_build_stop (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_stop (openldev); }

void openldev_view_browser (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_browser (openldev); }

void openldev_view_line_numbering (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_line_numbering (openldev); }

void openldev_view_output (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_output (openldev); }

void openldev_view_statusbar (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_statusbar (openldev); }

void openldev_view_toolbar (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_toolbar (openldev); }

void openldev_help_about (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_about (openldev); }

void openldev_help_documentation (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_documentation (openldev); }

void openldev_help_report_bug (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_report_bug (openldev); }
